# ANALYSIS OF LEADBEATER'S POSSUM DATA USING A CONDITIONAL POISSON MODEL - trunc.poisson and
# fit.func FUNCTIONS WRITTEN BY SIMON BARRY

poss <- read.table("data.txt",sep="\t",header=T,row.names=NULL,col.names=c("no.lb","stags","slope","baa","shrubs","bark"))
attach(poss)

lstags <- log(stags+1)


# FITTING THE TRUNCATED POISSON DISTRIBUTION TO THE NON-ZERO COMPONENT OF THE 
# CONDITIONAL POISSON MODEL
"trunc.poisson"<-
function(x, y, beta = NULL, iter = 10, tol = 1.0000000000000001e-05)
{
	done <- F
	if(is.null(beta)) {
		temp.frame <- data.frame(y99 = y, x99 = x)
		init.model <- glm(y99 ~ ., family = poisson(), data = 
			temp.frame)
		curr.bet <- coef(init.model)
	}
	else curr.bet <- beta
	count <- 0
	while(!done) {
		count <- count + 1
		curr.exp <- exp(x %*% curr.bet)
		trunc.exp <- curr.exp/(1 - exp( - curr.exp))
		curr.var <- as.vector((curr.exp - curr.exp^2 * exp( - curr.exp) -
			curr.exp * exp( - curr.exp))/((1 - exp( - curr.exp))^2))
		score <- t(x) %*% (y - trunc.exp)
		inf <- t(x * curr.var) %*% x
		new.bet <- curr.bet + solve(inf) %*% score
		change <- mean(abs(new.bet - curr.bet))
		curr.bet <- new.bet
		if(change < tol)
			done <- T
		if(iter < count)
			done <- T
		loglikmu <- -1 * sum(exp(x %*% curr.bet)) + sum(y * (x %*% 
			curr.bet)) - sum(lgamma(y + 1)) - sum(log(1 - exp( - 
			exp(x %*% curr.bet))))
		logliky <- -1 * sum(y) + sum(y * log(y)) - sum(lgamma(y + 1)) - 
			sum(log(1 - exp(-y)))
		mydev <- 2 * (logliky - loglikmu)
	}
	list(coefs = curr.bet, inf = inf, fitted = exp(x %*% curr.bet)/(1 - exp(
		 - exp(x %*% curr.bet))), resid = (y - (exp(x %*% curr.bet)/(1 - 
		exp( - exp(x %*% curr.bet)))))/sqrt(as.vector((curr.exp - 
		curr.exp^2 * exp( - curr.exp) - curr.exp * exp( - curr.exp))/((
		1 - exp( - curr.exp))^2))), loglik = loglikmu)
}


# FITTING THE CONDITIONAL POISSON MODEL
"fit.func"<-
function(response, cov01, covabund, ...)
{
	temp.data <- data.frame(y1 = as.numeric(response > 0), y0 = as.numeric(
		response == 0), term = cov01)
	mod1 <- glm(cbind(y1, y0) ~ -1 + ., family = binomial(), data = 
		temp.data, x = T)
	ind <- response > 0
	mod2 <- trunc.poisson(covabund[ind,  ], response[ind], ...)
	list(mod01 = mod1, modabund = mod2)
}


x <- cbind(1,lstags,baa)
z <- cbind(1,lstags,bark,shrubs,slope)
out <- fit.func(no.lb,x,z) 


# PRINT OUT THE ESTIMATES AND STANDARD ERRORS FOR THE PRESENCE/ABSENCE COMPONENT
summary(out$mod01)
PAest <- as.vector(out$mod01$coefficients)


# PRINT OUT THE ESTIMATES AND STANDARD ERRORS FOR THE NON-ZERO COMPONENT
TPest <- as.vector(out$modabund$coefs)
TPse <- as.vector(sqrt(diag(solve(out$modabund$inf))))
cbind(TPest,TPse)


# CALCULATE THE LOG-LIKELIHOOD FOR THE CONDITIONAL POISSON MODEL
"loglik" <-
function(p,lambda,y)
{
        n <- length(y)
        loglikb <- 0
        loglika <- 0
        for (i in c(1:n)) {
                if (y[i]==0) loglikb <- loglikb + log(1 - p[i])
                else {
                        loglikb <- loglikb + log(p[i])
                        loglika <- loglika + y[i] * log(lambda[i]) - lambda[i] -
                                log(gamma(y[i] + 1)) - log(1 - exp(-lambda[i]))
                }
        }
        sum <- loglikb + loglika
        sum
}

p <- as.vector(exp(x %*% PAest)/(1+exp(x %*% PAest)))
lambda <- as.vector(exp(z %*% TPest))
ll <- loglik(p,lambda,no.lb)
print(paste("The log-likelihood for the conditional Poisson model is",ll))

q()
